﻿#define CANCEL
#define PROCESSCANCELLATION

using System;
using System.Threading;
using System.Threading.Tasks;

namespace NewInDotNet4
{
    public class CancelTask
    {

        private static int Sum(CancellationToken ct, int n)
        {
            Int32 sum = 0;
            for (; n > 0; n--)
            {
                Thread.Sleep(5);
#if PROCESSCANCELLATION
              if (ct.IsCancellationRequested)  // hier overbodig, aangezien ThrowIfCancellationRequested dit ook oproept
              {
                    ct.ThrowIfCancellationRequested();  
                    // throws a System.OperationCanceledException if ct has had cancellation requested
                    // (TaskCanceledException is a subclass of OperationCanceledException)
                    // state of task = Cancelled
                    // (when a task does not throw an exception due to a cancellation and just finishes,
                    // the state of the task becomes RanToCompletion)
              }
#endif
                checked { sum += n; }
            }
            return sum;
        }

        public static void Main()
        {
                CancellationTokenSource cts = new CancellationTokenSource();
                Task<int> t = new Task<int>(() => Sum(cts.Token, 1000), cts.Token);
                t.Start();
                Thread.Sleep(1); 
#if CANCEL
                cts.Cancel();  // communicates a request for cancellation
#endif
                try
                {
                    // If the task got canceled, Result (or Wait) will throw an AggregateException
                     Console.WriteLine("The sum is: " + t.Result);   // An int value
                }
                catch (AggregateException ae)
                {
                    Console.WriteLine(ae);
                    // ae.Handle(e => e is OperationCanceledException);  // /????
                    Console.WriteLine("Sum was canceled");
                }
                Console.WriteLine("State of task: "+t.Status);
        }
    }
}