﻿/* PLINQ */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace NewInDotNet4
{
    class Program10
    {
        static void Main(string[] args)
        {
            IEnumerable<int> getallen = Enumerable.Range(1, 100000);

            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();

            List<int> evenGetallen = (from getal in getallen
                                      //where getal % 2 == 0 && getal % 3 == 0  // niet complex genoeg om voordeel te halen uit parallele verwerking :-(
                                      where IsOk(getal)
                                      select getal).ToList<int>();

            stopwatch.Stop();
            Console.WriteLine("time LINQ query (sequential) in milliseconds: {0}", stopwatch.ElapsedMilliseconds);

            foreach (int getal in evenGetallen)
            {
                //Console.WriteLine(getal);
            }

            stopwatch.Reset();
            stopwatch.Start();

            /* When you write a query, opt in to PLINQ by invoking the ParallelEnumerable.AsParallel 
             * extension method on the data source
             */
            List<int> evenGetallenParallel = (from getal in getallen.AsParallel()
                                              //  where getal % 2 == 0 && getal % 3 == 0 
                                              where IsOk(getal)
                                              select getal).ToList<int>();

            /*
             * By default, PLINQ is conservative. 
             * At run time, the PLINQ infrastructure analyzes the overall structure of the query. 
             * If the query is likely to yield speedups by parallelization, 
             * PLINQ partitions the source sequence into tasks that can be run concurrently. 
             * If it is not safe to parallelize a query, PLINQ just runs the query sequentially. 
             * If PLINQ has a choice between a potentially expensive parallel algorithm 
             * or an inexpensive sequential algorithm, it chooses the sequential algorithm by default. 
             * You can use the WithExecutionMode<TSource> method and the System.Linq.ParallelExecutionMode enumeration to instruct PLINQ to select the parallel algorithm.
             */

            stopwatch.Stop();
            Console.WriteLine("time PLINQ query (parallel) in milliseconds: {0}", stopwatch.ElapsedMilliseconds);
        }


        private static bool IsOk(int getal)
        {
            // time consuming action nabootsen
            for (int i = 0; i < 10000; i++) { }
            return getal % 2 == 0 && getal % 3 == 0;
        }
    }
}
