﻿/*  data parallelisme: Parallel.ForEach */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;

namespace NewInDotNet4
{
    class ParallelForeach
    {
        static void Main(string[] args)
        {
            //  voor .net 4:sequentiele foreach
            IList<Person> personen = new List<Person>();
            personen.Add(new Person() { FirstName = "Mickey", Name = "Mouse" });
            personen.Add(new Person() { FirstName = "Donald", Name = "Duck" });
            personen.Add(new Person() { FirstName = "Daisy", Name = "Duck" });


            Console.WriteLine("Executing sequential loop...");
            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();
            foreach (Person person in personen)
            {
                Process(person);
            }
            stopwatch.Stop();
            Console.WriteLine("Sequential loop time in milliseconds: {0}", stopwatch.ElapsedMilliseconds);
            
            Console.WriteLine();

            stopwatch.Reset();
            Console.WriteLine("Executing parallel loop...");
            stopwatch.Start();
            // sinds .net4: parallele uitvoering foreach
            Parallel.ForEach(personen, person => Process(person));
            stopwatch.Stop();
            Console.WriteLine("Parallel loop time in milliseconds: {0}", stopwatch.ElapsedMilliseconds);

        }

        private static void Process(Person person)
        {
            person.Name = person.Name.ToUpper();
            for (int i = 0; i < 10000000; i++)
            {
                // om effect van parallele uitvoering te testen
            }
           // Console.WriteLine("{0} {1}", person.FirstName, person.Name);  // synchronous calls to shared resources like the Console or File System will significantly degrade performance of a parallel loop
        }
    }
    class Person
    {
        public String Name { get; set; }
        public String FirstName { get; set; }
    }
}
