/* Multitasking */

/* in .net 4 is de namespace System.Threading.Tasks toegevoegd
 * Deze bevat klassen die het werken met parallele taken vereenvoudigen (door de klasse Task)
 * 
 * Een TAAK is een hoeveelheid werk die uitgevoerd moet worden.
 * De effectieve uitvoering v/e taak gebeurt door een thread;
 * maar het doet er niet toe welke thread de taak uitvoert.
 * (de CLR zal bepalen welke thread een taak uitvoert 
 * (een thread kan meerdere taken uitvoeren))
 * 
 * Creatie en starten van een taak komt overeen met het gebruik v/d methode
 * ThreadPool.QueueUserWorkItem.
 * D.w.z. dat de taak in de threadpool geplaatst wordt en wacht tot een threadpoolthread beschikbaar is om ze uit te voeren.
 * 
 * O.a. in volgende gevallen is het gebruik van gewone threads i.p.v. tasks nodig:
 * foreground thread nodig (i.e. thread die blijft lopen als mainthread gestopt is)
 * thread moet prioriteit krijgen
 * "stabiele" identiteit of naam van thread nodig 
 */

#define WAITFORTASKS

using System;
using System.Threading;
using System.Threading.Tasks;

namespace TaskTester
{  
   class TaskTester
   {
      static void Main( string[] args )
      {
          // Maak 2 impliciete taken en voer ze parallel uit
          // (Parallel.Invoke voert de opgegeven actions indien mogelijk parallel uit)
          Action actie = () => new MessagePrinter().Print();
          Console.WriteLine("Invoking parallel actions");
          Parallel.Invoke(actie, () => new MessagePrinter().Print());
          Console.WriteLine("\nParallel actions invoked\n");

         // Maak 2 expliciete taken
         MessagePrinter printer1 = new MessagePrinter();
         Task task1 = (new Task(() => printer1.Print()));

         MessagePrinter printer2 = new MessagePrinter();
         Task task2 = new Task(() => printer2.Print());

         Console.WriteLine( "Starting tasks" );

         // Roep de Start method op van elke task
         task1.Start();
         task2.Start();

        // Maak en start een 3de taak
         Task task3 = Task.Factory.StartNew(() => new MessagePrinter().Print());
         Console.WriteLine( "Tasks started\n" ); 

#if WAITFORTASKS
	    Console.WriteLine("WaitAll waits for tasks to finish");
        // wachten tot een reeks taken klaar is
        Task.WaitAll(task1, task2);
        Console.WriteLine("Wait waits for a specific task to finish");
        // wachten tot een bepaalde taak klaar is
        task3.Wait();
#endif

        // result v/e taak opvragen
        Task<int> task4 = Task.Factory.StartNew( () => {
            Console.WriteLine("task4 started");
            for (int i = 0; i < 100000000; i++) {}
            Thread.Sleep(3000);
            Console.WriteLine("task4 completed");
            return new Random().Next(); }
            );
        Console.WriteLine("Result van task4 = {0}", task4.Result);

		Console.WriteLine( "End Main" ); 
        // task stopt als calling thread stopt 
	
      } // einde method Main

   } // einde class TaskTester

   // De Print methode van deze klasse wordt uitgevoerd door de taken
   class MessagePrinter 
   {
      private int sleepTime;
      private static Random random = new Random();
      
      // constructor
      public MessagePrinter()
      {
         // Neem random sleep time tussen 0 and 5 seconde
		 sleepTime = random.Next( 5001 );
      }

      // methode Print controleert de taken
      public void Print() 
      {
         // Laat de taak slapen voor een bepaalde tijd
         Console.WriteLine( "taak met id " +
            Task.CurrentId + " slaapt voor " + sleepTime +" milliseconden." );

         Thread.Sleep(sleepTime);

         // toon gegevens van taak 
         Console.WriteLine( "Taak met id "+ Task.CurrentId + " is wakker" );

      } // einde methode Print

   } // einde class MessagePrinter
}


