﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace vbdragdrop
{
    public partial class Form1 : Form
    {

        public Form1()
        {
            InitializeComponent();
            treeView1.Nodes.Clear();
            VulTreeView();
            // we tonen alle nodes uitgeklapt.
            treeView1.ExpandAll();
            // De treeview wordt de target van de drop
            // => we moeten toelaten dat we items droppen in de treeview
            treeView1.AllowDrop = true;
            //Het slepen begint met een itemdrag event in de source
            treeView1.ItemDrag += new ItemDragEventHandler(treeView1_ItemDrag);
            //Het DragEnter event treedt op wanneer we in de target binnenkomen
            treeView1.DragEnter += new DragEventHandler(treeView1_DragEnter);
            //Het DragOver event treedt op wanneer we bewegen in de target
            treeView1.DragOver += new DragEventHandler(treeView1_DragOver);
            //Het DragDrop event treedt op wanneer we het item loslaten bover de target
            treeView1.DragDrop += new DragEventHandler(treeView1_DragDrop);
        }

        void treeView1_DragDrop(object sender, DragEventArgs e)
        {
            //altijd controleren of de juiste gegevens aanwezig zijn
            if (e.Data.GetDataPresent(typeof(TreeNode)))
            {
                TreeNode subnode = (TreeNode)e.Data.GetData(typeof(TreeNode));
                if (subnode.Parent != treeView1.SelectedNode)
                {
                    //Haal de node op, verwijder de node bij de huidige parent
                    //en voeg toe aan de geselecteerde parent
                    subnode.Parent.Nodes.Remove(subnode);
                    treeView1.SelectedNode.Nodes.Add(subnode);
                }
            }
        }
        void treeView1_DragOver(object sender, DragEventArgs e)
        {
            //kijk over welke node de muis "hovert"
            Point muisPositie = treeView1.PointToClient(new Point(e.X, e.Y));
            TreeNode node = treeView1.GetNodeAt(muisPositie);
            //wanneer het één van de hoofdnodes is
            //=> zet die node als de geselecteerde node zodat
            // de gebruiker ziet waar het gedragde item zal gedropt worden
            if (node.Parent == null)
            {
                treeView1.SelectedNode = node;
            }
        }

        void treeView1_DragEnter(object sender, DragEventArgs e)
        {
            //Wanneer we over de Treeview "hoveren" tijdens een drag
            //willen we een move-cursor zien
            //Via e.KeyState kunnen we zien welke SHIFT/ALT/CTRL knoppen 
            //muisbutton de gebruiker heeft ingedrukt
            //Via e.AllowedEffects kunnen we zien wat we mogen doen met het item
            //(dat werd bepaald in ItemDrag)
            e.Effect = DragDropEffects.Move;
        }


        void treeView1_ItemDrag(object sender, ItemDragEventArgs e)
        {
            //We starten een Drag alleen wanneer één van de subnodes
            //wordt versleept.
            TreeNode node = e.Item as TreeNode;
            if (node.Parent != null)
            {
                treeView1.DoDragDrop(e.Item, DragDropEffects.Move);
            }
        }
        private void VulTreeView()
        {
            TreeNode node = treeView1.Nodes.Add("Groep 1");
            node.Nodes.Add("Initiatie .NET");
            node.Nodes.Add("C# inleiding");
            node.Nodes.Add("C# vervolmaking");
            node = treeView1.Nodes.Add("Groep 2");
            node.Nodes.Add("UML");
            node.Nodes.Add("UML gevalstudies");
            node.Nodes.Add("Design patterns");
            node = treeView1.Nodes.Add("Groep 3");
            node.Nodes.Add("Theorie databanken");
            node.Nodes.Add("SQL");
        }

        
    }
}
